@@ -16,6 +16,8 @@
 # along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 import logging
+from typing import Any
+from typing import Optional
 
 from gi.repository import GLib
 
@@ -45,7 +47,7 @@ from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import Mode
 from nbxmpp.dispatcher import StanzaDispatcher
 from nbxmpp.util import get_stream_header
-from nbxmpp.util import get_stanza_id
+from nbxmpp.util import generate_id
 from nbxmpp.util import Observable
 from nbxmpp.util import validate_stream_header
 from nbxmpp.util import LogAdapter
@@ -149,6 +151,10 @@ class Client(Observable):
     def features(self):
         return self._stream_features
 
+    @property
+    def resumeable(self):
+        return self._smacks.resumeable
+
     @property
     def sm_supported(self):
         return self._smacks.sm_supported
@@ -278,11 +284,9 @@ class Client(Observable):
 
     @property
     def mechs(self):
-        return set(self._allowed_mechs or set(['SCRAM-SHA-256',
+        return set(self._allowed_mechs or set(['SCRAM-SHA-512',
+                                               'SCRAM-SHA-256',
                                                'SCRAM-SHA-1',
-                                               'GSSAPI',
-                                               # 'SCRAM-SHA-256-PLUS',
-                                               # 'SCRAM-SHA-1-PLUS',
                                                'PLAIN']))
 
     def set_mechs(self, mechs):
@@ -475,8 +479,8 @@ class Client(Observable):
         else:
             self._set_error(StreamError.CONNECTION_FAILED,
                             'connection-failed',
-                            (f'Unable to connect to last '
-                             'successful address: {self._current_address}'))
+                            ('Unable to connect to last '
+                             f'successful address: {self._current_address}'))
             self.notify('connection-failed')
 
     def _disconnect_with_error(self, error_domain, error, text=None):
@@ -569,8 +573,13 @@ class Client(Observable):
         GLib.source_remove(self._ping_source_id)
         self._ping_source_id = None
 
-    def send_stanza(self, stanza, now=False, callback=None,
-                    timeout=None, user_data=None):
+    def send_stanza(self,
+                    stanza: Protocol,
+                    now: bool = False,
+                    callback: Optional[Any] = None,
+                    timeout: Optional[int] = None,
+                    user_data: Optional[Any] = None) -> str:
+
         if user_data is not None and not isinstance(user_data, dict):
             raise ValueError('arg user_data must be of dict type')
 
@@ -579,7 +588,7 @@ class Client(Observable):
 
         id_ = stanza.getID()
         if id_ is None:
-            id_ = get_stanza_id()
+            id_ = generate_id()
             stanza.setID(id_)
 
         if callback is not None:
@@ -636,6 +645,11 @@ class Client(Observable):
                 # other connection methods if an error happensafterwards
                 self._connect_successful = True
 
+            if self._stream_authenticated and self._mode.is_login_test:
+                self.notify('login-successful')
+                self.disconnect()
+                return
+
             self.state = StreamState.WAIT_FOR_FEATURES
 
         elif self.state == StreamState.WAIT_FOR_FEATURES:
@@ -678,14 +692,6 @@ class Client(Observable):
 
         elif self.state == StreamState.AUTH_SUCCESSFUL:
             self._stream_authenticated = True
-            if self._mode.is_login_test:
-                self.notify('login-successful')
-                # Reset parser because we will receive a new stream header
-                # which will otherwise lead to a parsing error
-                self._dispatcher.reset_parser()
-                self.disconnect()
-                return
-
             self._start_stream()
 
         elif self.state == StreamState.AUTH_FAILED:
