@@ -42,7 +42,7 @@ class UserAvatar(BaseModule):
         'publish': 'PubSub',
         'request_item': 'PubSub',
         'request_items': 'PubSub',
-        'delete': 'PubSub',
+        'purge': 'PubSub',
     }
 
     def __init__(self, client):
@@ -80,7 +80,7 @@ class UserAvatar(BaseModule):
             return
 
         try:
-            data = AvatarMetaData.from_node(metadata)
+            data = AvatarMetaData.from_node(metadata, item.getAttr('id'))
         except Exception as error:
             self._log.warning('Malformed user avatar data: %s', error)
             self._log.warning(stanza)
@@ -128,7 +128,7 @@ class UserAvatar(BaseModule):
         if not metadata.getChildren():
             yield task.set_result(None)
 
-        yield AvatarMetaData.from_node(metadata)
+        yield AvatarMetaData.from_node(metadata, item.getAttr('id'))
 
     @iq_request_task
     def set_avatar(self, avatar, public=False):
@@ -141,7 +141,7 @@ class UserAvatar(BaseModule):
             result = yield self._publish_avatar_metadata(None, access_model)
             raise_if_error(result)
 
-            result = yield self.delete(Namespace.AVATAR_DATA)
+            result = yield self.purge(Namespace.AVATAR_DATA)
             yield finalize(task, result)
 
         result = yield self._publish_avatar(avatar, access_model)
@@ -159,7 +159,6 @@ class UserAvatar(BaseModule):
 
         for info, data in avatar.pubsub_avatar_info():
             item = _make_avatar_data_node(data)
-            self._log.info('Publish avatar data: %s, %s', info, access_model)
 
             result = yield self.publish(Namespace.AVATAR_DATA,
                                         item,
@@ -178,8 +177,6 @@ class UserAvatar(BaseModule):
     def _publish_avatar_metadata(self, metadata, access_model):
         task = yield
 
-        self._log.info('Publish avatar meta data: %s', metadata)
-
         options = {
             'pubsub#persist_items': 'true',
             'pubsub#access_model': access_model,
@@ -190,7 +187,7 @@ class UserAvatar(BaseModule):
 
         result = yield self.publish(Namespace.AVATAR_METADATA,
                                     metadata.to_node(),
-                                    id_='current',
+                                    id_=metadata.default,
                                     options=options,
                                     force_node_options=True)
 
@@ -223,13 +220,13 @@ def _get_avatar_data(item, id_):
         raise MalformedStanzaError('data node empty', item)
 
     try:
-        avatar = b64decode(data, return_type=bytes)
+        avatar = b64decode(data)
     except Exception as error:
         raise MalformedStanzaError(f'decoding error: {error}', item)
 
     avatar_sha = hashlib.sha1(avatar).hexdigest()
     if avatar_sha != id_:
-        raise MalformedStanzaError(f'avatar does not match sha', item)
+        raise MalformedStanzaError('avatar does not match sha', item)
 
     return AvatarData(data=avatar, sha=avatar_sha)
 
@@ -310,9 +307,10 @@ class AvatarData:
 @dataclass
 class AvatarMetaData:
     infos: List[AvatarInfo] = field(default_factory=list)
+    default: AvatarInfo = None
 
     @classmethod
-    def from_node(cls, node):
+    def from_node(cls, node, default=None):
         infos = []
         info_nodes = node.getTags('info')
         for info in info_nodes:
@@ -324,10 +322,12 @@ class AvatarMetaData:
                 height=info.getAttr('height'),
                 width=info.getAttr('width')
             ))
-        return cls(infos=infos)
+        return cls(infos=infos, default=default)
 
-    def add_avatar_info(self, avatar_info):
+    def add_avatar_info(self, avatar_info, make_default=False):
         self.infos.append(avatar_info)
+        if make_default:
+            self.default = avatar_info.id
 
     def to_node(self):
         return _make_metadata_node(self.infos)
@@ -342,7 +342,14 @@ class Avatar:
     metadata: AvatarMetaData = field(default_factory=AvatarMetaData)
     data: Dict[AvatarInfo, bytes] = field(init=False, default_factory=dict)
 
-    def add_image_source(self, data, type_, height, width, url=None):
+    def add_image_source(self,
+                         data,
+                         type_,
+                         height,
+                         width,
+                         url=None,
+                         make_default=True):
+
         sha = hashlib.sha1(data).hexdigest()
         info = AvatarInfo(bytes=len(data),
                           id=sha,
@@ -350,7 +357,7 @@ class Avatar:
                           height=height,
                           width=width,
                           url=url)
-        self.metadata.add_avatar_info(info)
+        self.metadata.add_avatar_info(info, make_default=make_default)
         self.data[info] = AvatarData(data=data, sha=sha)
 
     def pubsub_avatar_info(self):
