@@ -35,7 +35,7 @@ class _Namespaces:
     BOB: str = 'urn:xmpp:bob'
     BOOKMARKS: str = 'storage:bookmarks'
     BOOKMARKS_1: str = 'urn:xmpp:bookmarks:1'
-    BOOKMARKS_COMPAT: str = 'urn:xmpp:bookmarks:0#compat'
+    BOOKMARKS_COMPAT: str = 'urn:xmpp:bookmarks:1#compat'
     BOOKMARKS_COMPAT_PEP: str = 'urn:xmpp:bookmarks:1#compat-pep'
     BOOKMARK_CONVERSION: str = 'urn:xmpp:bookmarks-conversion:0'
     BROWSE: str = 'jabber:iq:browse'
@@ -62,6 +62,7 @@ class _Namespaces:
     DOMAIN_BASED_NAME: str = 'urn:xmpp:domain-based-name:1'
     EME: str = 'urn:xmpp:eme:0'
     ENCRYPTED: str = 'jabber:x:encrypted'
+    FASTEN: str = 'urn:xmpp:fasten:0'
     FILE_METADATA: str = 'urn:xmpp:file:metadata:0'
     FORWARD: str = 'urn:xmpp:forward:0'
     FRAMING: str = 'urn:ietf:params:xml:ns:xmpp-framing'
@@ -99,6 +100,8 @@ class _Namespaces:
     LOCATION: str = 'http://jabber.org/protocol/geoloc'
     MAM_1: str = 'urn:xmpp:mam:1'
     MAM_2: str = 'urn:xmpp:mam:2'
+    MESSAGE_MODERATE: str = 'urn:xmpp:message-moderate:0'
+    MESSAGE_RETRACT: str = 'urn:xmpp:message-retract:0'
     MOOD: str = 'http://jabber.org/protocol/mood'
     MSG_HINTS: str = 'urn:xmpp:hints'
     MUCLUMBUS: str = 'https://xmlns.zombofant.net/muclumbus/search/1.0'
@@ -129,6 +132,7 @@ class _Namespaces:
     PUBSUB_EVENT: str = 'http://jabber.org/protocol/pubsub#event'
     PUBSUB_OWNER: str = 'http://jabber.org/protocol/pubsub#owner'
     PUBSUB_PUBLISH_OPTIONS: str = 'http://jabber.org/protocol/pubsub#publish-options'
+    PUBSUB_NODE_MAX: str = 'http://jabber.org/protocol/pubsub#config-node-max'
     RECEIPTS: str = 'urn:xmpp:receipts'
     REGISTER: str = 'jabber:iq:register'
     REGISTER_FEATURE: str = 'http://jabber.org/features/iq-register'
@@ -154,6 +158,7 @@ class _Namespaces:
     STREAM_MGMT: str = 'urn:xmpp:sm:3'
     STYLING: str = 'urn:xmpp:styling:0'
     TIME_REVISED: str = 'urn:xmpp:time'
+    TIME: str = 'urn:xmpp:time'
     TLS: str = 'urn:ietf:params:xml:ns:xmpp-tls'
     TUNE: str = 'http://jabber.org/protocol/tune'
     URL_DATA: str = 'http://jabber.org/protocol/url-data'
@@ -167,6 +172,7 @@ class _Namespaces:
     XHTML: str = 'http://www.w3.org/1999/xhtml'
     XMPP_STREAMS: str = 'urn:ietf:params:xml:ns:xmpp-streams'
     X_OOB: str = 'jabber:x:oob'
+    XRD: str = 'http://docs.oasis-open.org/ns/xri/xrd-1.0'
 
 
-Namespace = _Namespaces()
+Namespace: _Namespaces = _Namespaces()
