@@ -18,12 +18,20 @@ data structures, including jabber-objects like JID or different stanzas and
 sub- stanzas) handling routines
 """
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Union
+from typing import Optional
+from typing import cast
+
 import time
 import hashlib
 import functools
 import warnings
 from base64 import b64encode
-from collections import namedtuple
+from dataclasses import dataclass
+from dataclasses import asdict
 
 from gi.repository import GLib
 
@@ -36,6 +44,7 @@ def ascii_upper(s):
     return s.upper()
 
 SASL_AUTH_MECHS = [
+    'SCRAM-SHA-512',
     'SCRAM-SHA-256-PLUS',
     'SCRAM-SHA-256',
     'SCRAM-SHA-1-PLUS',
@@ -240,27 +249,6 @@ _errorcodes = {
     'cancel': 'invalid-from'
 }
 
-_status_conditions = {
-    'realjid-public': 100,
-    'affiliation-changed': 101,
-    'unavailable-shown': 102,
-    'unavailable-not-shown': 103,
-    'configuration-changed': 104,
-    'self-presence': 110,
-    'logging-enabled': 170,
-    'logging-disabled': 171,
-    'non-anonymous': 172,
-    'semi-anonymous': 173,
-    'fully-anonymous': 174,
-    'room-created': 201,
-    'nick-assigned': 210,
-    'banned': 301,
-    'new-nick': 303,
-    'kicked': 307,
-    'removed-affiliation': 321,
-    'removed-membership': 322,
-    'removed-shutdown': 332,
-}
 
 _localpart_disallowed_chars = set('"&\'/:<>@')
 _localpart_escape_chars = ' "&\'/:<>@'
@@ -489,7 +477,7 @@ def deprecation_warning(message):
 
 
 @functools.lru_cache(maxsize=None)
-def validate_localpart(localpart):
+def validate_localpart(localpart: str) -> str:
     if not localpart or len(localpart.encode()) > 1023:
         raise LocalpartByteLimit
 
@@ -504,7 +492,7 @@ def validate_localpart(localpart):
 
 
 @functools.lru_cache(maxsize=None)
-def validate_resourcepart(resourcepart):
+def validate_resourcepart(resourcepart: str) -> str:
     if not resourcepart or len(resourcepart.encode()) > 1023:
         raise ResourcepartByteLimit
 
@@ -516,7 +504,7 @@ def validate_resourcepart(resourcepart):
 
 
 @functools.lru_cache(maxsize=None)
-def validate_domainpart(domainpart):
+def validate_domainpart(domainpart: str) -> str:
     if not domainpart:
         raise DomainpartByteLimit
 
@@ -540,12 +528,12 @@ def validate_domainpart(domainpart):
 
 
 @functools.lru_cache(maxsize=None)
-def idna_encode(domain):
+def idna_encode(domain: str) -> str:
     return idna.encode(domain, uts46=True).decode()
 
 
 @functools.lru_cache(maxsize=None)
-def escape_localpart(localpart):
+def escape_localpart(localpart: str) -> str:
     # https://xmpp.org/extensions/xep-0106.html#bizrules-algorithm
     #
     # If there are any instances of character sequences that correspond
@@ -567,7 +555,7 @@ def escape_localpart(localpart):
 
 
 @functools.lru_cache(maxsize=None)
-def unescape_localpart(localpart):
+def unescape_localpart(localpart: str) -> str:
     if localpart.startswith('\\20') or localpart.endswith('\\20'):
         # Escaped JIDs are not allowed to start or end with \20
         # so this localpart must be already unescaped
@@ -584,29 +572,31 @@ def unescape_localpart(localpart):
     return localpart
 
 
-class JID(namedtuple('JID',
-                     ['jid', 'localpart', 'domain', 'resource'])):
+@dataclass(frozen=True)
+class JID:
+    localpart: Optional[str] = None
+    domain: Optional[str] = None
+    resource: Optional[str] = None
 
-    __slots__ = []
-
-    def __new__(cls, jid=None, localpart=None, domain=None, resource=None):
-        if jid is not None:
-            deprecation_warning('JID(jid) is deprecated, use from_string()')
-            return JID.from_string(str(jid))
+    def __init__(self,
+                 localpart: Optional[str] = None,
+                 domain: Optional[str] = None,
+                 resource: Optional[str] = None):
 
         if localpart is not None:
             localpart = validate_localpart(localpart)
+            object.__setattr__(self, "localpart", localpart)
 
         domain = validate_domainpart(domain)
+        object.__setattr__(self, "domain", domain)
 
         if resource is not None:
             resource = validate_resourcepart(resource)
-
-        return super().__new__(cls, None, localpart, domain, resource)
+            object.__setattr__(self, "resource", resource)
 
     @classmethod
     @functools.lru_cache(maxsize=None)
-    def from_string(cls, jid_string):
+    def from_string(cls, jid_string: str, force_bare: bool = False) -> JID:
         # https://tools.ietf.org/html/rfc7622#section-3.2
 
         # Remove any portion from the first '/' character to the end of the
@@ -625,14 +615,18 @@ class JID(namedtuple('JID',
         else:
             localpart, domainpart = None, rest
 
-        return cls(jid=None,
-                   localpart=localpart,
+        if force_bare:
+            if localpart is None:
+                raise LocalpartByteLimit
+            resourcepart = None
+
+        return cls(localpart=localpart,
                    domain=domainpart,
                    resource=resourcepart)
 
     @classmethod
     @functools.lru_cache(maxsize=None)
-    def from_user_input(cls, user_input, escaped=False):
+    def from_user_input(cls, user_input: str, escaped: bool = False) -> JID:
         # Use this if we want JIDs to be escaped according to XEP-0106
         # The standard JID parsing cannot be applied because user_input is
         # not a valid JID.
@@ -659,16 +653,15 @@ class JID(namedtuple('JID',
             localpart = None
             domainpart = user_input
 
-        return cls(jid=None,
-                   localpart=localpart,
+        return cls(localpart=localpart,
                    domain=domainpart,
                    resource=None)
 
-    def __str__(self):
+    def __str__(self) -> str:
         if self.localpart:
             jid = f'{self.localpart}@{self.domain}'
         else:
-            jid = self.domain
+            jid = cast(str, self.domain)
 
         if self.resource is not None:
             return f'{jid}/{self.resource}'
@@ -677,55 +670,59 @@ class JID(namedtuple('JID',
     def __hash__(self):
         return hash(str(self))
 
-    def __eq__(self, other):
+    def __eq__(self, other: Union[str, JID]) -> bool:
         if isinstance(other, str):
-            deprecation_warning('comparing string with JID is deprected')
             try:
                 return JID.from_string(other) == self
             except Exception:
                 return False
-        return super().__eq__(other)
 
-    def __ne__(self, other):
+        return (self.localpart == other.localpart and
+                self.domain == other.domain and
+                self.resource == other.resource)
+
+    def __ne__(self, other: Union[str, JID]) -> bool:
         return not self.__eq__(other)
 
-    def domain_to_ascii(self):
+    def domain_to_ascii(self) -> str:
         return idna_encode(self.domain)
 
     @property
-    def bare(self):
+    def bare(self) -> str:
         if self.localpart is not None:
             return f'{self.localpart}@{self.domain}'
         return self.domain
 
     @property
-    def is_bare(self):
+    def is_bare(self) -> bool:
         return self.resource is None
 
-    def new_as_bare(self):
+    def new_as_bare(self) -> JID:
         if self.resource is None:
             return self
-        return self._replace(resource=None)
+        return JID.from_string(self.bare)
 
-    def bare_match(self, other):
+    def bare_match(self, other: Union[str, JID]) -> bool:
         if isinstance(other, str):
             other = JID.from_string(other)
         return self.bare == other.bare
 
     @property
-    def is_domain(self):
+    def is_domain(self) -> bool:
         return self.localpart is None and self.resource is None
 
     @property
-    def is_full(self):
+    def is_full(self) -> bool:
         return (self.localpart is not None and
                 self.domain is not None and
                 self.resource is not None)
 
-    def new_with(self, **kwargs):
-        return self._replace(**kwargs)
+    def new_with(self, **kwargs: dict[str, str]) -> JID:
+        new = asdict(self)
+        new.update(kwargs)
+        return JID(**new)
 
-    def to_user_string(self, show_punycode=True):
+    def to_user_string(self, show_punycode: bool = True) -> str:
         domain = self.domain_to_ascii()
         if domain.startswith('xn--') and show_punycode:
             domain_encoded = f' ({domain})'
@@ -741,63 +738,16 @@ class JID(namedtuple('JID',
             return f'{localpart}@{self.domain}{domain_encoded}'
         return f'{localpart}@{self.domain}/{self.resource}{domain_encoded}'
 
-    def bareMatch(self, other):
-        deprecation_warning('bareMatch() is deprected use bare_match()')
-        return self.bare_match(other)
-
-    @property
-    def isBare(self):
-        deprecation_warning('isBare() is deprected use '
-                            'the attribute is_bare')
-        return self.is_bare
-
-    @property
-    def isDomain(self):
-        deprecation_warning('isDomain() is deprected use '
-                            'the attribute is_domain')
-        return self.is_domain
-
-    @property
-    def isFull(self):
-        deprecation_warning('isFull() is deprected use '
-                            'the attribute is_full')
-        return self.is_full
-
-    def copy(self):
+    def copy(self) -> JID:
         deprecation_warning('copy() is not needed, JID is immutable')
         return self
 
-    def getNode(self):
-        deprecation_warning('getNode() is deprected use '
-                            'the attribute localpart')
-        return self.localpart
-
-    def getDomain(self):
-        deprecation_warning('getDomain() is deprected use '
-                            'the attribute domain')
-        return self.domain
-
-    def getResource(self):
-        deprecation_warning('getResource() is deprected use '
-                            'the attribute resource')
-        return self.resource
-
-    def getStripped(self):
-        deprecation_warning('getStripped() is deprected use '
-                            'the attribute bare')
-        return self.bare
-
-    def getBare(self):
-        deprecation_warning('getBare() is deprected use '
-                            'the attribute bare')
-        return self.bare
-
 
 class StreamErrorNode(Node):
     def __init__(self, node):
         Node.__init__(self, node=node)
 
-        self._text = {}
+        self._text: dict[Optional[str], str] = {}
 
         text_elements = self.getTags('text', namespace=Namespace.XMPP_STREAMS)
         for element in text_elements:
@@ -805,14 +755,14 @@ class StreamErrorNode(Node):
             text = element.getData()
             self._text[lang] = text
 
-    def get_condition(self):
+    def get_condition(self) -> Optional[str]:
         for tag in self.getChildren():
             if (tag.getName() != 'text' and
                     tag.getNamespace() == Namespace.XMPP_STREAMS):
                 return tag.getName()
         return None
 
-    def get_text(self, pref_lang=None):
+    def get_text(self, pref_lang: Optional[str] = None) -> str:
         if pref_lang is not None:
             text = self._text.get(pref_lang)
             if text is not None:
@@ -945,13 +895,13 @@ class Protocol(Node):
             return JID.from_string(attr)
         return attr
 
-    def getID(self):
+    def getID(self) -> Optional[str]:
         """
         Return the value of the 'id' attribute
         """
         return self.getAttr('id')
 
-    def setTo(self, val):
+    def setTo(self, val: Union[str, JID]):
         """
         Set the value of the 'to' attribute
         """
@@ -959,13 +909,13 @@ class Protocol(Node):
             val = JID.from_string(val)
         self.setAttr('to', val)
 
-    def getType(self):
+    def getType(self) -> Optional[str]:
         """
         Return the value of the 'type' attribute
         """
         return self.getAttr('type')
 
-    def setFrom(self, val):
+    def setFrom(self, val: Union[str, JID]):
         """
         Set the value of the 'from' attribute
         """
@@ -973,19 +923,19 @@ class Protocol(Node):
             val = JID.from_string(val)
         self.setAttr('from', val)
 
-    def setType(self, val):
+    def setType(self, val: str):
         """
         Set the value of the 'type' attribute
         """
         self.setAttr('type', val)
 
-    def setID(self, val):
+    def setID(self, val: str):
         """
         Set the value of the 'id' attribute
         """
         self.setAttr('id', val)
 
-    def getError(self):
+    def getError(self) -> Optional[str]:
         """
         Return the error-condition (if present) or the textual description
         of the error (otherwise)
@@ -1096,7 +1046,11 @@ class Protocol(Node):
                 props.append(prop)
         return props
 
-    def getTag(self, name, attrs=None, namespace=None, protocol=False):
+    def getTag(self,
+               name: str,
+               attrs: Optional[dict[str, Any]] = None,
+               namespace: Optional[str] = None,
+               protocol: bool = False) -> Optional[Node]:
         """
         Return the Node instance for the tag.
         If protocol is True convert to a new Protocol/Message instance.
@@ -1108,14 +1062,14 @@ class Protocol(Node):
             return Protocol(node=tag)
         return tag
 
-    def __setitem__(self, item, val):
+    def __setitem__(self, item: str, val: Union[str, JID]):
         """
         Set the item 'item' to the value 'val'
         """
         if item in ['to', 'from']:
             if not isinstance(val, JID):
                 val = JID.from_string(val)
-        return self.setAttr(item, val)
+        self.setAttr(item, val)
 
 
 class Message(Protocol):
@@ -1344,7 +1298,7 @@ class Presence(Protocol):
         """
         Return the status string of the message
         """
-        return self.getTagData('status')
+        return self.getTagData('status') or ''
 
     def setPriority(self, val):
         """
@@ -1436,14 +1390,14 @@ class Iq(Protocol):
     """
 
     def __init__(self,
-                 typ=None,
-                 queryNS=None,
-                 attrs=None,
-                 to=None,
-                 frm=None,
-                 payload=None,
-                 xmlns=Namespace.CLIENT,
-                 node=None):
+                 typ: Optional[str] = None,
+                 queryNS: Optional[str] = None,
+                 attrs: Optional[dict[str, str]] = None,
+                 to: Optional[Union[JID, str]] = None,
+                 frm: Optional[Union[JID, str]] = None,
+                 payload: Optional[list[Union[Node, str]]] = None,
+                 xmlns: str = Namespace.CLIENT,
+                 node: Optional[Node] = None):
         """
         You can specify type, query namespace any additional attributes,
         recipient of the iq, sender of the iq, any additional payload (f.e.
@@ -1465,7 +1419,7 @@ class Iq(Protocol):
         if queryNS:
             self.setQueryNS(queryNS)
 
-    def getQuery(self):
+    def getQuery(self) -> Optional[Node]:
         """
         Return the IQ's child element if it exists, None otherwise.
         """
@@ -1475,7 +1429,7 @@ class Iq(Protocol):
             return children[0]
         return None
 
-    def getQueryNS(self):
+    def getQueryNS(self) -> Optional[str]:
         """
         Return the namespace of the 'query' child element
         """
@@ -1484,7 +1438,7 @@ class Iq(Protocol):
             return tag.getNamespace()
         return None
 
-    def getQuerynode(self):
+    def getQuerynode(self) -> Optional[str]:
         """
         Return the 'node' attribute value of the 'query' child element
         """
@@ -1493,16 +1447,7 @@ class Iq(Protocol):
             return tag.getAttr('node')
         return None
 
-    def getQueryPayload(self):
-        """
-        Return the 'query' child element payload
-        """
-        tag = self.getQuery()
-        if tag:
-            return tag.getPayload()
-        return None
-
-    def getQueryChildren(self):
+    def getQueryChildren(self) -> Optional[list[Node]]:
         """
         Return the 'query' child element child nodes
         """
@@ -1511,7 +1456,7 @@ class Iq(Protocol):
             return tag.getChildren()
         return None
 
-    def getQueryChild(self, name=None):
+    def getQueryChild(self, name: Optional[str] = None) -> Optional[Node]:
         """
         Return the 'query' child element with name, or the first element
         which is not an error element
@@ -1528,7 +1473,7 @@ class Iq(Protocol):
                     return node
         return None
 
-    def setQuery(self, name=None):
+    def setQuery(self, name: Optional[str] = None) -> Node:
         """
         Change the name of the query node, creating it if needed. Keep the
         existing name if none is given (use 'query' if it's a creation).
@@ -1541,25 +1486,25 @@ class Iq(Protocol):
             query.setName(name)
         return query
 
-    def setQueryNS(self, namespace):
+    def setQueryNS(self, namespace: str) -> None:
         """
         Set the namespace of the 'query' child element
         """
         self.setQuery().setNamespace(namespace)
 
-    def setQueryPayload(self, payload):
+    def setQueryPayload(self, payload: list[Union[Node, str]]):
         """
         Set the 'query' child element payload
         """
         self.setQuery().setPayload(payload)
 
-    def setQuerynode(self, node):
+    def setQuerynode(self, node: str) -> None:
         """
         Set the 'node' attribute value of the 'query' child element
         """
         self.setQuery().setAttr('node', node)
 
-    def buildReply(self, typ):
+    def buildReply(self, typ: str) -> Iq:
         """
         Build and return another Iq object of specified type. The to, from and
         query child node of new Iq are pre-set as reply to this Iq.
@@ -1571,7 +1516,7 @@ class Iq(Protocol):
         iq.setQuery(self.getQuery().getName()).setNamespace(self.getQueryNS())
         return iq
 
-    def buildSimpleReply(self, typ):
+    def buildSimpleReply(self, typ: str) -> Iq:
         return Iq(typ,
                   to=self.getFrom(),
                   attrs={'id': self.getID()})
@@ -1698,13 +1643,15 @@ class Hashes2(Node):
 
 
 class BindRequest(Iq):
-    def __init__(self, resource):
-        if resource is not None:
-            resource = Node('resource', payload=resource)
+    def __init__(self, resource: Optional[str]):
+        if resource is None:
+            res = resource
+        else:
+            res = Node('resource', payload=resource)
         Iq.__init__(self, typ='set')
         self.addChild(node=Node('bind',
                                 {'xmlns': Namespace.BIND},
-                                payload=resource))
+                                payload=res))
 
 
 class TLSRequest(Node):
@@ -1719,7 +1666,7 @@ class SessionRequest(Iq):
 
 
 class StreamHeader(Node):
-    def __init__(self, domain, lang=None):
+    def __init__(self, domain: str, lang: Optional[str] = None):
         if lang is None:
             lang = 'en'
         Node.__init__(self,
@@ -1732,7 +1679,7 @@ class StreamHeader(Node):
 
 
 class WebsocketOpenHeader(Node):
-    def __init__(self, domain, lang=None):
+    def __init__(self, domain: str, lang: Optional[str] = None):
         if lang is None:
             lang = 'en'
         Node.__init__(self,
@@ -1748,7 +1695,7 @@ class WebsocketCloseHeader(Node):
 
 
 class Features(Node):
-    def __init__(self, node):
+    def __init__(self, node: Node):
         Node.__init__(self, node=node)
 
     def has_starttls(self):
@@ -1761,7 +1708,7 @@ class Features(Node):
     def has_sasl(self):
         return self.getTag('mechanisms', namespace=Namespace.SASL) is not None
 
-    def get_mechs(self):
+    def get_mechs(self) -> set[str]:
         mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL)
         if mechanisms is None:
             return set()
@@ -1847,24 +1794,18 @@ class Error(Protocol):
         If the 'node' is not the received stanza but locally created ('to' and
         'from' fields needs not swapping) specify the 'reply' argument as false.
         """
+        if node.getType() == 'error':
+            raise ValueError('Can’t create error reply from error')
+
         if reply:
             Protocol.__init__(self,
                               to=node.getFrom(),
-                              frm=node.getTo(),
                               node=node)
+            self.delAttr('from')
         else:
             Protocol.__init__(self, node=node)
         self.setError(error)
-        if node.getType() == 'error':
-            self.__str__ = self.__dupstr__
 
-    def __dupstr__(self, _dup1=None, _dup2=None):
-        """
-        Dummy function used as preventor of creating error node in reply to
-        error node. I.e. you will not be able to serialise "double" error
-        into string.
-        """
-        return ''
 
 class DataField(Node):
     """
